@@ -1,10 +1,10 @@
 from __future__ import unicode_literals
-from collections import OrderedDict, MutableMapping
+from collections import OrderedDict, MutableMapping, namedtuple
 from io import BytesIO
 from unicodedata import normalize
 from xml.etree import cElementTree as ET
-import contextlib
 import posixpath
+import sys
 import zipfile
 
 
@@ -21,8 +21,13 @@ The UCF spec is included with the Creative Suite SDK. Also here:
 """
 
 
-__version__ = '0.2'
-
+__version__ = '0.2.1'
+__all__ = [
+    'UCF',
+    'BadFileFormat',
+    'Rootfile',
+    'element_tostring',
+]
 
 # Python 3 compatibility
 try:
@@ -30,11 +35,10 @@ try:
 except NameError:
     unichr, unicode = chr, str
 
-
 META_INF = 'META-INF'
 MIME_TYPE = 'mimetype'
-UTF8 = 'UTF-8'
-ASCII = 'ASCII'
+UTF8 = 'utf-8'
+ASCII = 'us-ascii'
 CONTAINER = 'container.xml'
 SEP = posixpath.sep
 
@@ -59,22 +63,14 @@ NSMAP = {
 }
 
 
-def element_tostring(ele, xml_declaration=True, encoding=UTF8,
-                     default_namespace=None):
-    out = BytesIO()
-    # Prevents ETree writing unicode to stream when add <?xml> header
-    encoding = encoding.encode(UTF8)
-    
-    tree = ET.ElementTree(ele)
-    tree.write(out, xml_declaration=xml_declaration, encoding=encoding,
-        default_namespace=default_namespace)
-    return out.getvalue()
-
-
 for prefix in NSMAP:
     ET.register_namespace(prefix, NSMAP[prefix])
     
 
+# Represents a <rootfile full-path="" media-type="" /> element
+Rootfile = namedtuple('Rootfile', 'path mimetype')
+
+
 class BadFileFormat(Exception):
     """The archive does not conform to the UCF specification."""
 
@@ -101,7 +97,7 @@ class MetaFilesDict(MutableMapping):
                 yield key[len(self._meta_path):]
     
     def __len__(self):
-        return len(list(self.keys()))
+        return len(list(iter(self)))
         
 
 class UCF(OrderedDict):
@@ -129,8 +125,7 @@ class UCF(OrderedDict):
         return OrderedDict.__setitem__(self, key, val)
         
     def open(self):
-        # zipfile didn't get context manager support until Python 2.7
-        with contextlib.closing(zipfile.ZipFile(self._filename)) as archive:
+        with zipfile.ZipFile(self._filename) as archive:
             for info in archive.infolist():
                 # Python 3 gives us unicode member names already
                 name = _decode(info.filename)
@@ -162,7 +157,7 @@ class UCF(OrderedDict):
         for name in all_names:
             _assert_valid_name(name)
                 
-        with contextlib.closing(zipfile.ZipFile(filename, mode='w')) as archive:
+        with zipfile.ZipFile(filename, mode='w') as archive:
             # First must be mimetype and it must be without compression
             archive.writestr(MIME_TYPE, self[MIME_TYPE])
             all_names.remove(MIME_TYPE)
@@ -188,6 +183,16 @@ class UCF(OrderedDict):
         return object.__repr__(self)
 
 
+def element_tostring(ele, xml_declaration=True, encoding=UTF8,
+                     default_namespace=None):
+    out = BytesIO()
+    # Use str(encoding) as suggested by http://bugs.python.org/issue15811#msg169395.
+    tree = ET.ElementTree(ele)
+    tree.write(out, xml_declaration=xml_declaration, encoding=str(encoding),
+        default_namespace=default_namespace)
+    return out.getvalue()
+
+
 def _encode(string, encoding=UTF8):
     if isinstance(string, unicode):
         return string.encode(encoding)
@@ -211,12 +216,14 @@ def _build_container(names_and_types):
     container.set('{%s}version' % NSMAP['container'], '1.0')
     rootfiles = ET.SubElement(container, '{%s}rootfiles' % NSMAP['container'])
     for name, media_type in names_and_types:
-        # We require unicode, but let you use bytes
-        name, media_type = _decode(name), _decode(media_type)
-        
+        # We require unicode, but let you use bytes, hence the _decode()
         rootfile = ET.SubElement(rootfiles, '{%s}rootfile' % NSMAP['container'])
-        rootfile.set('{%s}full-path' % NSMAP['container'], name)
-        rootfile.set('{%s}media-type' % NSMAP['container'], media_type)
+        rootfile.set('{%s}full-path' % NSMAP['container'], _decode(name))
+        
+        # media-type is an optional attribute
+        if media_type:
+            media_type = _decode(media_type)
+            rootfile.set('{%s}media-type' % NSMAP['container'], media_type)
     return element_tostring(container, default_namespace=NSMAP['container'])    
     
     
@@ -233,7 +240,8 @@ def _read_rootfiles(xml):
     eles = tree.findall('./{%(od)s}rootfiles/{%(od)s}rootfile' % {'od': NSMAP['container']})
     pairs = [(ele.get('full-path'), ele.get('media-type')) for ele in eles]
     # Python 2/3 ET returns bytes on 2, unicode on 3
-    return [(_decode(fp), _decode(mt)) for fp, mt in pairs]
+    # The media-type attribute is optional per spec. Guard against None.
+    return [Rootfile(_decode(fp), _decode(mt) if mt else mt) for fp, mt in pairs]
 
 
 def _assert_valid_name(name):
